require_relative 'config_file'

module NonConfig
  #
  # Base class you need to inherit
  #
  class Base
    ENVIRONMENTS = [:development, :test, :production].freeze
    @@files = {}
    @@env = nil

    #
    # Loads file.
    #
    # Files that were added last are considered to be more important
    # i.e. if you have
    #
    #     source :file1, '...'
    #     source :file2, '...'
    #     source :file3, '...'
    #
    # all operations will be executed from file3 to file1
    #
    # @param file_alias [Symbol, String]  alias for file
    # @param path [String] full path to config
    # @raise [StandardError]
    #
    def self.source(file_alias, path)
      fail('file alias is already taken') if @@files[file_alias]

      @@files[file_alias] = ConfigFile.new(path)
    end

    def self.config_file(file_alias = nil)
      file_alias = @@files.to_a.last && @@files.to_a.last[0] unless file_alias
      fail FileNotRegistered unless file_alias && @@files.key?(file_alias)

      @@files[file_alias]
    end

    # sets environment. See Base::ENVIRONMENTS
    def self.env(env)
      return @@env = nil unless env

      env = env.to_sym
      fail UnsupportedEnvironment unless ENVIRONMENTS.include? env

      @@env = env.to_s
    end

    # Sets value to one of config files
    def self.set(key, value, file_alias = nil)
      file = config_file file_alias

      if @@env
        file[@@env] ||= {}
        file[@@env][key] = value
      else
        file[key] = value
      end
    end

    # alias for #set key, value
    def self.[]=(key, value)
      set(key, value)
    end

    #
    # Tries to find object.
    #
    # Unlike Base.key it won't raise exception on not found
    #
    # @return nil if not found
    #
    def self.[](key)
      default = nil
      @@files.reverse_each do |_name, file|
        default = file[key] if default.nil?

        return file[@@env][key] if file[@@env] && file[@@env][key]
        return file[key] if !@@env && file[key]
      end

      default
    end

    # If `args` empty it will try to find `m.to_s` and `m` in config
    # else (or if not found) it will call super
    def self.method_missing(m, *args)
      result = self[m.to_s]
      return result if result

      result = self[m]
      return result if result

      super
    end

    def self.save(file_alias = nil)
      fail FileNotRegistered if file_alias && !@@files.key?(file_alias)

      if file_alias
        @@files[file_alias].save
      else
        @@files.values.each(&:save)
      end
    end
  end
end
